<?php
/*
Plugin Name: GitHub Stars Widget
Plugin URI: https://brettterpstra.com/projects/github-stars/
Description: A WordPress widget to display your GitHub stars. Nothing fancy.
Version: 1.0.1
Author: ttscoff
Author URI: https://brettterpstra.com
 */
/*  Copyright 2021 Brett Terpstra

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License (GPL v2) only.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

class Stars_Widget {

	public function __construct() {
		add_action('admin_head', array($this, 'widget_error_css'));
		add_action('widgets_init', array($this, 'register_widget'));
	}

	public function widget_error_css() {

		if (!is_admin()) {
			return;
		}

		echo '<style type="text/css">span.stars_error_message {color:#cd0000;padding-top:5px;display:block;text-align:right;font-weight:bold;}</style>';
	}

	public function register_widget() {
		register_widget('bt_ListStarsWidget');
	}

	public static function fetch_stars($user = '', $number = 0) {

		if (false === $stars = get_transient('github_stars_' . $user)) {

			$data = wp_remote_get('https://api.github.com/users/' . urlencode($user) . '/starred?&per_page=' . $number);

			if (is_wp_error($data)) {
				set_transient('github_stars_' . $user, '', HOUR_IN_SECONDS);
				return false;
			}

			$stars = json_decode($data['body']);

			if (empty($stars)) {
				set_transient('github_stars_' . $user, '', HOUR_IN_SECONDS);
				return false;
			}

			set_transient('github_stars_' . $user, $stars, HOUR_IN_SECONDS);
		}

		return $stars;
	}
}

$Stars_Widget = new Stars_Widget();

class bt_ListStarsWidget extends WP_Widget {

	function __construct() {
		$widget_ops = array('classname' => 'bt_github_stars', 'description' => __('Displays a list of stars hosted on GitHub'));
		parent::__construct('bt_github_stars', __('GitHub Stars'), $widget_ops);
		$this->alt_option_name = 'bt_github_stars';
	}

	function widget($args, $instance) {

		if (empty($instance['github_user'])) {
			return;
		}

		$user = $instance['github_user'];
		$number = !empty($instance['stars_num']) && $instance['stars_num'] < 101 ? absint($instance['stars_num']) : 10;

		if (false === $stars = Stars_Widget::fetch_stars($user, $number)) {
			return;
		}

		$gh_link = 'https://github.com/' . $user . '?tab=stars';

		echo $args['before_widget'];
		$title = empty($instance['title']) ? '' : apply_filters('widget_title', $instance['title']);

		if (!empty($title)) {echo $args['before_title'] . $title . $args['after_title'];};

		echo '<dl class="stars_list">';

		foreach ($stars as $star) {

			$owner = $star->owner->login;
			$name = $star->name;
			$desc = !empty($star->description) ? $star->description : '';
			$url = !empty($star->html_url) ? $star->html_url : '';

			if (!empty($url) && !empty($name)) {

				echo '<dt class="star_title">';
				echo '<a href="' . esc_url($url) . '" target="_blank">&#9733; <span class="star_owner">' . esc_attr($owner) . '</span><span class="star_slash">/</span><strong class="star_name">' . esc_attr($name) . '</strong></a>';
				echo '</dt>';
			}

			if (!empty($desc)) {
				echo '<dd class="star_description">';
				echo esc_attr($desc);
				echo '</dd>';
			}
		}

		echo '</dl>';

		echo "<p class=\"stars_link\">&rarr; See all of <a href=\"$gh_link\">$user's stars on GitHub</a></p>";

		echo $args['after_widget'];
	}

	function update($new_instance, $old_instance) {

		$instance = $old_instance;

		$instance['title'] = sanitize_text_field($new_instance['title']);
		$instance['github_user'] = sanitize_text_field($new_instance['github_user']);
		$instance['stars_num'] = absint($new_instance['stars_num']);

		delete_transient('github_stars_' . $instance['github_user']);

		return $instance;
	}

	function form($instance) {

		$title = !empty($instance['title']) ? esc_attr($instance['title']) : '';
		$user = !empty($instance['github_user']) ? esc_attr($instance['github_user']) : '';
		$number = !empty($instance['stars_num']) ? absint($instance['stars_num']) : 10;
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Widget Title');?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		</p>

		<p>
			<label for="<?php echo $this->get_field_id('github_user'); ?>"><?php _e('GitHub username');?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('github_user'); ?>" name="<?php echo $this->get_field_name('github_user'); ?>" type="text" value="<?php echo esc_attr($user); ?>" />
			<?php if (empty($user)) {echo '<span class="star_error_message">' . __('Username is required!') . '</span>';}?>
		</p>

		<p>
			<label for="<?php echo $this->get_field_id('stars_num'); ?>"><?php _e('Stars to display');?></label>
			<input class="widefat" id="<?php echo $this->get_field_id('stars_num'); ?>" name="<?php echo $this->get_field_name('stars_num'); ?>" type="text" value="<?php echo esc_attr($number); ?>" />
		</p>
	<?php }

}
